#!/usr/bin/env python
# coding=utf-8

#from storage_constants import *
from classes import *

import logging

TIMESTMP_FMT = ("%H:%M:%S") # no %f for msecs
FORMAT = '%(asctime)s.%(msecs)03d %(name)-5s: %(message)s'
logging.basicConfig(format=FORMAT, datefmt=TIMESTMP_FMT, level=logging.DEBUG)

logger = logging.getLogger('Storage')


dsids = set()

def intern_DSID(dsid):
    assert(type(dsid) == list)
    for _dsid in dsids:
        if _dsid.dsid == dsid:
            return _dsid
    new = DSID(dsid)
    dsids.add(new)
    return new

devices_by_id = {}
devices = []

def get_device(deviceId):
    if not known_device(deviceId):
        return None
    return devices_by_id[deviceId]
def add_device(device):
    logger.info("add_device %s", device)
    devices_by_id[device.deviceId] = device
    devices.append(device)
def known_device(deviceId):
    logger.debug("known_device %d", deviceId)
    return devices_by_id.has_key(deviceId)

zones = []

def known_zone(zoneid):
    assert(type(zoneid) == int)
    match = [ zone for zone in zones if zone.zoneid == zoneid]
    return len(match) != 0
def get_zone(zoneid):
    assert(type(zoneid) == int)
    match = [ zone for zone in zones if zone.zoneid == zoneid]
    if (len(match) == 0):
        return None
    assert(len(match) == 1)
    return match[0]
def add_zone(zone):
    assert isinstance(zone, Zone)
    if known_zone(zone.zoneid):
        raise Exception("duplicate zone %s", str(zone))
    return zones.append(zone)
def del_zone(zoneid):
    assert(type(zoneid) == int)
    if not known_zone(zoneid):
        raise Exception("inexistant zone %s", str(zone))
    zones.remove(get_zone(zoneid))
def save_zone(zone):
    assert isinstance(zone, Zone)
    if known_zone(zone.zoneid):
        del_zone(zone.zoneid)
    add_zone(zone)
def all_zones():
    return list(zones)


# state must not be saved persistently -> default to inactive
dsms = []
dsms_by_id = {} # list is not a key

def known_dsm(dsmid):
    assert(type(dsmid) == list)
    match = [ dsm for dsm in dsms if dsm.dsid == dsmid]
    return len(match) != 0
def get_dsm(dsmid):
    assert(type(dsmid) == list)
    match = [ dsm for dsm in dsms if dsm.dsid == dsmid]
    if (len(match) == 0):
        return None
    assert(len(match) == 1)
    return match[0]
def del_dsm(dsmid):
    assert(type(dsmid) == list)
    if not known_dsm(dsmid):
        raise Exception("inexistant dsm %s", str(dsm))
    dsms.remove(get_dsm(dsmid))
def add_dsm(dsm):
    assert isinstance(dsm, DSMeter)
    if known_dsm(dsm.dsid):
        raise Exception("duplicate dsm %s", str(dsm))
    dsms.append(dsm)
def save_dsm(dsm):
    assert isinstance(dsm, DSMeter)
    if known_dsm(dsm.dsid):
        del_dsm(dsm.dsid)
    add_dsm(dsm)
def all_dsms():
    return list(dsms)


group_infos = []

def filter_GroupInfo_group(groupId):
    return [ info for info in group_infos if info.groupId == groupId ]
def filter_GroupInfo_zone_group_dsmid(zoneId, groupId, dsmid):
    return [ info for info in group_infos if (info.groupId == groupId and
                                              info.zoneId == zoneId and
                                              info.dsmid == dsmid) ]
def add_GroupInfo(groupInfo):
    match = filter_GroupInfo_zone_group_dsmid(groupInfo.zoneId,
                                              groupInfo.groupId,
                                              groupInfo.dsmid)
    assert(len(match) < 2)
    if len(match) == 1:
        group_infos.remove(match[0])
    group_infos.append(groupInfo)
def get_GroupInfo(groupId):
    return filter_GroupInfo_group(groupId)
def save_GroupInfo(groupInfo):
    add_GroupInfo(groupInfo)


if __name__=="__main__":

    foo = range(12)
    bar = range(12)
    assert(intern_DSID(foo) == intern_DSID(bar))
    assert(intern_DSID(foo) != DSID(bar))

    class device:
        def __init__(self, deviceId):
            self.deviceId = deviceId
        def __repr__(self):
            return "fake dS device %d" % self.deviceId

    add_device(device(933))
    add_device(device(547))
    add_device(device(248))

    assert (get_device(1023) == None)
    assert (not get_device(1023))

    class dsm(DSMeter):
        def __init__(self, dsmid):
            self.dsid = dsmid
        def __repr__(self):
            return "fake dS meter %r" % self.dsmid

    add_dsm(dsm([0x0, 0x1]))
    assert(known_dsm([0x0, 0x1]))
    assert(get_dsm([0x0, 0x1]).dsid == [0x0, 0x1])

    dsm1 = dsm([0x0, 0x2])
    add_dsm(dsm1)
    save_dsm(dsm1)
    del_dsm(dsm1.dsid)

    class fZone(Zone):
        def __init__(self, zoneid):
            #Zone.__init__(self, zoneId, 7, 4, )
            self.zoneid = zoneid

    zone1 = fZone(7)

    add_zone(zone1)
    assert(known_zone(zone1.zoneid))
    save_zone(zone1)
    del_zone(zone1.zoneid)


    class GroupInfo(object):
        def __init__(self, zoneId, groupId, dsmid):
            self.zoneId = zoneId
            self.groupId = groupId
            self.dsmid = dsmid

    # group present in several zones, represented via multiple GroupInfo
    gInfo5_21 = GroupInfo(5, 21, [0x0, 0x1])
    gInfo7_21 = GroupInfo(7, 21, [0x0, 0x2])
    #get_GroupInfos(21)
    add_GroupInfo(gInfo5_21)  # group-21 present in zone 5
    add_GroupInfo(gInfo7_21)
    assert(len(get_GroupInfo(21)) == 2)
