#!/usr/bin/env python
from ds485_const import *
from ds485.dsuid import *
from notification_queue import *
from process_pkt import split_pkt
from util import list2hexstr

import asyncore, socket, select
from threading import Thread
import time
import traceback, signal
import logging
from Queue import Queue

BUFFER_SIZE = 1024

TIMESTMP_FMT = ("%H:%M:%S") # no %f for msecs
FORMAT = '%(asctime)s.%(msecs)03d %(name)-5s %(message)s'
logging.basicConfig(format=FORMAT, datefmt=TIMESTMP_FMT, level=logging.INFO)

class State:
    def __init__(self, port):
        self.connect = BusState.DS485_STATE_ACTIVE
        self.dsmids = [intern_DSUID(createDsmId()),
                       intern_DSUID(createDsmId())]
        self.dsuid = intern_DSUID(createDssId())
        self.port = port

class Connection(asyncore.dispatcher):
    def __init__(self, connect, desc, state):
        asyncore.dispatcher.__init__(self, connect)
        self.log = logging.getLogger("Connection--%s" % desc)
        self.log.setLevel(logging.INFO)
        self.log.info("__init__")
        self.state = state
        self.queue = Queue()
        self.outbuf = []
        self.buf = []

    def __del__(self):
        log = self.log
        log.info("__del__")
        # TODO remove from server

    def handle_connect(self):
        log = self.log
        log.info("handle_connect")
        notification_queue.put((handle_connect, None, self))

    def handle_close(self):
        log = self.log
        log.info("handle_close")
        self.close()

    def handle_read(self):
        log = self.log
        log.debug("handle_read")
        data = self.recv(8192)
        if not data:
            return
        buf = self.buf
        buf.extend(map(ord, list(data)))
        for pkt in split_pkt(buf):
            notification_queue.put((handle_read, pkt, self))
        #assert(len(buf) == 0)

    def writable(self):
        log = self.log
        ret = (len(self.outbuf) > 0 or (not self.queue.empty()))
        if ret:
            log.debug("writable %r", ret)
        return ret

    def handle_write(self):
        log = self.log
        if ((len(self.outbuf) == 0) and self.queue.empty()):
            return

        if not self.outbuf:
            self.outbuf = self.queue.get()
        sent = self.send(self.outbuf)
        log.debug("handle_write sent:%r pkt:%r queued:%d", sent, len(self.outbuf),
                 not self.queue.empty())
        self.outbuf = self.outbuf[sent:]

    def push(self, pkt):
        log = self.log
        log.debug("handle_push len %d", len(pkt))
        self.queue.put(pkt)


class Daemon(asyncore.dispatcher):
    def __init__(self, port, state):
        asyncore.dispatcher.__init__(self)
        self.log = logging.getLogger("Daemon-listen-%d" % port)
        #self.log.setLevel(logging.DEBUG)
        self.log.info("init")
        self.port = port
        self.ct = 0
        self.state = state
        self.clients = []
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.set_reuse_addr()
        self.bind(('localhost', port))
        self.listen(5)

    def send_all_clients(self, pkt):
        self.log.debug("send_all_clients %s", list2hexstr(pkt))
        for c in self.clients:
            c.push(pkt)

    def close_all(self):
        self.log.debug("close_all")
        for c in self.clients:
            c.close()
        self.clients = []
        self.close()

    def handle_connect(self):
        self.log.info('handle_connect')

    def dump_device_list(self):
        log = self.log
        s = self.state
        log.info("device list daemon-%d:", s.port)
        for dsm in s.dsmids:
            log.info("\t%s", dsm)

    def handle_accept(self):
        try:
            log = self.log
            pair = self.accept()
            if pair is None:
                return
            sock, addr = pair
            log.info('Incoming connection from %s' % repr(addr))
            self.ct += 1
            client = Connection(sock, "%d-%d" % (self.port, self.ct), self.state)
            self.clients.append(client)
            notification_queue.put((handle_accept, None, self))
        except:
            print traceback.format_exc()


def async_loop(sec, count):
    log = logging.getLogger("AsyncCore")
    try:
        asyncore.loop(timeout=sec, count = count)
        #log.info("asyncore.loop iter")
    except select.error, e:
        log.info("select error %s", e)
    except  Exception, e:
        log.info("Error %s", type(e))
        print traceback.format_exc()

if __name__== "__main__":
    f = []
    f.append(Daemon(5017, State(5017)))
    f.append(Daemon(5018, State(5018)))

    async_loop(1, 3)

    print "port should be listening, now kill first deamon"
    f[0].close()
    del f[0]

    async_loop(3, 1000)
    print "done"
